{-----------------------------------------------------------------------------
 Unit Name: uFibonacci
 Author:    Rob Bracken
 Purpose:   Fibonacci sequence iterator
 History:
-----------------------------------------------------------------------------}


unit uFibonacci;

interface

uses
  uIIterator
  , sysutils
  ;

type

  EFibonacciException = class(Exception);

  TFibonacci = class(TInterfacedObject, IIterator)
  private

    FVal1: integer;
    FVal2: integer;

    // Implementations of the IIterator interface
    function HasNext: boolean;
    function HasPrev: boolean;
    function Current: integer;
    function Next: integer;
    function Prev: integer;

  public

    constructor Create;

  end;

implementation

const
  Num1: integer = 0;
  Num2: integer = 1;

{ TFibonacci }

constructor TFibonacci.Create;
begin

  inherited Create;
  FVal1 := Num1;
  FVal2 := Num2;

end;

function TFibonacci.Current: integer;
begin
  Result := FVal1;
end;

function TFibonacci.HasNext: boolean;
begin

  // Are we about to overflow?
  Result := (High(FVal2) - FVal2) > FVal1;

end;

function TFibonacci.HasPrev: boolean;
begin

  // Have we got back to the beginning?
  Result := not((FVal1 = Num1) and (FVal2 = Num2));

end;

function TFibonacci.Next: integer;
var
  nTemp: integer;
begin

  if HasNext then
  begin
    // Calculate the next value in the series
    nTemp := FVal2;
    FVal2 := FVal1 + FVal2;
    FVal1 := nTemp;
    Result := Current;
  end
  else
  begin
    raise EFibonacciException.Create('No more values');
  end;

end;

function TFibonacci.Prev: integer;
var
  nTemp: integer;
begin

  if HasPrev then
  begin
    // Calculate the previous value in the series
    nTemp := FVal1;
    FVal1 := FVal2 - FVal1;
    FVal2 := nTemp;
    Result := Current;
  end
  else
  begin
    raise EFibonacciException.Create('No more values');
  end;

end;

end.
