{-----------------------------------------------------------------------------
 Unit Name: ufrmMain
 Author:    Rob Bracken
 Purpose:   Demo program to go with the BUG More Iterators article
 History:
-----------------------------------------------------------------------------}


unit ufrmMain;

interface

uses
  Windows
  , Messages
  , SysUtils
//  , Variants                  // Commented out for Delphi 5
  , Classes
  , Graphics
  , Controls
  , Forms
  , Dialogs
  , uIIterator
  , uIApptIterator
  , uAppointments
  , ComCtrls
  , StdCtrls
  , Buttons
  , ExtCtrls
  ;

type
  TfrmMain = class(TForm)
    Edit1: TEdit;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    Label1: TLabel;
    ListBox1: TListBox;
    Button1: TButton;
    GroupBox1: TGroupBox;
    rbUp: TRadioButton;
    rbDown: TRadioButton;
    Button2: TButton;
    Memo1: TMemo;
    Label2: TLabel;
    ListBox2: TListBox;
    Label3: TLabel;
    CheckBox1: TCheckBox;
    Shape1: TShape;
    Shape2: TShape;
    Label4: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure ListBox2Click(Sender: TObject);
    procedure CheckBox1Click(Sender: TObject);
    procedure Label4Click(Sender: TObject);
  private
    { Private declarations }
    FFibonacci: IIterator;
    FFibonacci2: IIterator;
    FFibVal: integer;
    FFmtStr: string;
    FApptWho: TStringList;
    FApptWhoWith: TStringList;
    FAppointments: TAppointments;

    procedure DisplayAppointments(Iterator: IApptIterator);
    procedure SetFibVal(Value: integer);

  protected

    // Define a property so we can do the necessary when we get another
    // Fibonacci value
    property FibVal: integer
      read FFibVal
      write SetFibVal;

  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

uses
  uFibonacci
  , uApptIterator
  , uApptWhoIterator
  , uApptNoneIterator
  ;

{$R *.dfm}

procedure TfrmMain.FormCreate(Sender: TObject);
var
  nCounter: integer;
begin

  // Create the Fibonacci iterators
  FFibonacci := TFibonacci.Create;
  FFibonacci2 := TFibonacci.Create;

  // Get the first value in the Fibonacci series & display it
  FibVal := FFibonacci.Current;

  // Work out the format string for the Fibonacci list box
  FFmtStr := '%' + IntToStr(Length(IntToStr(High(integer)))) + 'd';

  // Create an Appointments list
  FAppointments := TAppointments.Create;

  // Create & load the people lists for appointments
  FApptWho := TStringList.Create;
  FApptWhoWith := TStringList.Create;
  FApptWho.CommaText := 'Scott, Virgil, Alan, Gordon, John, Geoff, Penelope, Brains';
  FApptWhoWith.CommaText := '"Mr. J. Austin", "Mr. K. Bentley", "Mr. L. Chrysler", "Mr. M. Daimler", "Mr. N. Ford", "Mr. M. Gee", "Mr. O. Humber"';

  // Create some random appointments
  for nCounter := 1 to 1000 do
    FAppointments.AddRandomAppointment(FApptWho, FApptWhoWith);

  // Populate the filter list box
  ListBox2.Items.Add('All');
  ListBox2.Items.Add('None');
  ListBox2.Items.AddStrings(FApptWho);

end;

procedure TfrmMain.SetFibVal(Value: integer);
begin
  // Save & display the new Fibonacci value
  FFibVal := Value;
  Edit1.Text := IntToStr(FFibVal);
end;

procedure TfrmMain.SpeedButton1Click(Sender: TObject);
begin
  // Get & display the next Fibonacci value
  FibVal := FFibonacci.Next;
end;

procedure TfrmMain.SpeedButton2Click(Sender: TObject);
begin
  // Get & display the previous Fibonacci value
  FibVal := FFibonacci.Prev;
end;

procedure TfrmMain.Button1Click(Sender: TObject);
begin
  // Exit the program
  Close;
end;

procedure TfrmMain.Button2Click(Sender: TObject);
begin
  // Display the next Fibonacci value in the list box
  // If this is the first entry, display the current value
  if (ListBox1.Items.Count = 0) then
  begin
    ListBox1.Items.Add(Format(FFmtStr, [FFibonacci2.Current]));
    Exit;
  end;
  // Get the next/prev value
  try
    if rbUp.Checked then
    begin
      // Next value
      ListBox1.Items.Add(Format(FFmtStr, [FFibonacci2.Next]));
    end
    else
    begin
      // Previous value
      ListBox1.Items.Add(Format(FFmtStr, [FFibonacci2.Prev]));
    end;
  except
    on E: EFibonacciException do;         // Ignore "eof/bof" error
  end;
end;

procedure TfrmMain.ListBox2Click(Sender: TObject);
var
  ApptIter: IApptIterator;
begin
  if (ListBox2.ItemIndex > -1) then
  begin
    // Change the "Who" string we're using to filter the appointments
    if (ListBox2.Items[ListBox2.ItemIndex] = 'All') then
    begin
      ApptIter := TApptIterator.Create(FAppointments);
    end
    else
    begin
      if (ListBox2.Items[ListBox2.ItemIndex] = 'None') then
      begin
        ApptIter := TApptNoneIterator.Create;
      end
      else
      begin
        ApptIter := TApptWhoIterator.Create(FAppointments,
                                                ListBox2.Items[ListBox2.ItemIndex]);
      end;
    end;
    DisplayAppointments(ApptIter);
  end;
end;

procedure TfrmMain.DisplayAppointments(Iterator: IApptIterator);
var
  slTemp: TStringList;
begin
  slTemp := TStringList.Create;
  try
    if CheckBox1.Checked then
    begin
      // Reverse order
      if Assigned(Iterator.Last) then
        slTemp.Add(Iterator.Current.AsString);
      while Iterator.HasPrev do
        slTemp.Add(Iterator.Prev.AsString);
    end
    else
    begin
      // Forward order
      if Assigned(Iterator.First) then
        slTemp.Add(Iterator.Current.AsString);
      while Iterator.HasNext do
        slTemp.Add(Iterator.Next.AsString);
    end;
    Memo1.Lines.Clear;
    Memo1.Lines.Add('Who' + #9 + 'With Whom' + #9 + 'When' + #9#9 + 'How Long');
    Memo1.Lines.Add('');
    Memo1.Lines.AddStrings(slTemp);
  finally
    slTemp.Free;
  end;
end;

procedure TfrmMain.CheckBox1Click(Sender: TObject);
begin
  // Redisplay the filtered appointments
  ListBox2Click(Sender);
end;

procedure TfrmMain.Label4Click(Sender: TObject);
begin
  CheckBox1.Checked := not CheckBox1.Checked;
end;

end.
